from dataclasses import dataclass
from pathlib import Path
from typing import Optional

from terality_serde import SerdeMixin


@dataclass
class TransferConfigLocal(SerdeMixin):
    path: Path


@dataclass
class SessionInfoLocal(SerdeMixin):
    id: str
    upload_config: TransferConfigLocal
    download_config: TransferConfigLocal


@dataclass
class TransferConfig(SerdeMixin):
    default_aws_region: str
    bucket: str
    key_prefix: str

    def bucket_region(self, aws_region: Optional[str] = None) -> str:
        return f'{self.bucket}-{self.default_aws_region if aws_region is None else aws_region}'


@dataclass
class SessionInfo(SerdeMixin):
    id: str
    upload_config: TransferConfig
    download_config: TransferConfig
